@@ -4,14 +4,19 @@ use strict;
 use warnings;
 use 5.010;
 
+use parent 'Class::Accessor';
+
 use Travel::Routing::DE::VRR::Route::Part;
 
-our $VERSION = '1.05';
+our $VERSION = '2.02';
+
+Travel::Routing::DE::VRR::Route->mk_ro_accessors(
+	qw(duration ticket_text ticket_type fare_adult fare_child vehicle_time));
 
 sub new {
-	my ( $obj, @parts ) = @_;
+	my ( $obj, $info, @parts ) = @_;
 
-	my $ref = {};
+	my $ref = $info;
 
 	for my $part (@parts) {
 		push(
@@ -47,7 +52,7 @@ Travel::Routing::DE::VRR::Route - Single route (connection) between two points
 
 =head1 VERSION
 
-version 1.05
+version 2.02
 
 =head1 DESCRIPTION
 
@@ -60,17 +65,34 @@ You usually want to acces it via C<< $efa->routes >>.
 
 =head1 METHODS
 
+=head2 ACCESSORS
+
 =over
 
-=item my $route = Travel::Routing::DE::VRR::Route->new(I<@parts>)
+=item $route->duration
 
-Creates a new Travel::Routing::DE::VRR::Route elements consisting of I<parts>,
-which are Travel::Routing::DE::VRR::Route::Part elements.
+route duration as string in HH:MM format
 
 =item $route->parts
 
 Returns a list of Travel::Routing::DE::VRR::Route::Part(3pm) elements describing
-the actual route.
+the actual route
+
+=item $route->ticket_type
+
+Type of the required ticket for this route, if available (empty string otherwise)
+
+=item $route->fare_adult
+
+ticket price for an adult in EUR
+
+=item $route->fare_child
+
+ticket price for a child in EUR
+
+=item $route->vehicle_time
+
+on-vehicle time (excluding waiting time) of the route in minutes
 
 =back
 
